# Autogenerated by nbdev

d = { 'settings': { 'branch': 'master',
                'doc_baseurl': '/onprem',
                'doc_host': 'https://amaiya.github.io',
                'git_url': 'https://github.com/amaiya/onprem',
                'lib_path': 'onprem'},
  'syms': { 'onprem.app.OnPrem': {},
            'onprem.app.console': {},
            'onprem.app.pages.1_Prompts': {},
            'onprem.app.pages.2_Document_QA': {},
            'onprem.app.pages.3_Document_Analysis': {},
            'onprem.app.pages.4_Document_Search': {},
            'onprem.app.pages.5_Manage': {},
            'onprem.app.utils': {},
            'onprem.hf.base': { 'onprem.hf.base.Pipeline': ('hf.base.html#pipeline', 'onprem/hf/base.py'),
                                'onprem.hf.base.Pipeline.batch': ('hf.base.html#pipeline.batch', 'onprem/hf/base.py')},
            'onprem.hf.data.base': { 'onprem.hf.data.base.Data': ('hf.data.base.html#data', 'onprem/hf/data/base.py'),
                                     'onprem.hf.data.base.Data.__call__': ('hf.data.base.html#data.__call__', 'onprem/hf/data/base.py'),
                                     'onprem.hf.data.base.Data.__init__': ('hf.data.base.html#data.__init__', 'onprem/hf/data/base.py'),
                                     'onprem.hf.data.base.Data.labels': ('hf.data.base.html#data.labels', 'onprem/hf/data/base.py'),
                                     'onprem.hf.data.base.Data.length': ('hf.data.base.html#data.length', 'onprem/hf/data/base.py'),
                                     'onprem.hf.data.base.Data.prepare': ('hf.data.base.html#data.prepare', 'onprem/hf/data/base.py'),
                                     'onprem.hf.data.base.Data.process': ('hf.data.base.html#data.process', 'onprem/hf/data/base.py')},
            'onprem.hf.data.labels': { 'onprem.hf.data.labels.Labels': ('hf.data.labels.html#labels', 'onprem/hf/data/labels.py'),
                                       'onprem.hf.data.labels.Labels.__init__': ( 'hf.data.labels.html#labels.__init__',
                                                                                  'onprem/hf/data/labels.py'),
                                       'onprem.hf.data.labels.Labels.process': ( 'hf.data.labels.html#labels.process',
                                                                                 'onprem/hf/data/labels.py')},
            'onprem.hf.data.questions': { 'onprem.hf.data.questions.Questions': ( 'hf.data.questions.html#questions',
                                                                                  'onprem/hf/data/questions.py'),
                                          'onprem.hf.data.questions.Questions.__init__': ( 'hf.data.questions.html#questions.__init__',
                                                                                           'onprem/hf/data/questions.py'),
                                          'onprem.hf.data.questions.Questions.answers': ( 'hf.data.questions.html#questions.answers',
                                                                                          'onprem/hf/data/questions.py'),
                                          'onprem.hf.data.questions.Questions.process': ( 'hf.data.questions.html#questions.process',
                                                                                          'onprem/hf/data/questions.py'),
                                          'onprem.hf.data.questions.Questions.tokenize': ( 'hf.data.questions.html#questions.tokenize',
                                                                                           'onprem/hf/data/questions.py')},
            'onprem.hf.data.sequences': { 'onprem.hf.data.sequences.Sequences': ( 'hf.data.sequences.html#sequences',
                                                                                  'onprem/hf/data/sequences.py'),
                                          'onprem.hf.data.sequences.Sequences.__init__': ( 'hf.data.sequences.html#sequences.__init__',
                                                                                           'onprem/hf/data/sequences.py'),
                                          'onprem.hf.data.sequences.Sequences.process': ( 'hf.data.sequences.html#sequences.process',
                                                                                          'onprem/hf/data/sequences.py')},
            'onprem.hf.data.texts': { 'onprem.hf.data.texts.Texts': ('hf.data.texts.html#texts', 'onprem/hf/data/texts.py'),
                                      'onprem.hf.data.texts.Texts.__init__': ( 'hf.data.texts.html#texts.__init__',
                                                                               'onprem/hf/data/texts.py'),
                                      'onprem.hf.data.texts.Texts.concat': ('hf.data.texts.html#texts.concat', 'onprem/hf/data/texts.py'),
                                      'onprem.hf.data.texts.Texts.process': ( 'hf.data.texts.html#texts.process',
                                                                              'onprem/hf/data/texts.py')},
            'onprem.hf.data.tokens': { 'onprem.hf.data.tokens.Tokens': ('hf.data.tokens.html#tokens', 'onprem/hf/data/tokens.py'),
                                       'onprem.hf.data.tokens.Tokens.__getitem__': ( 'hf.data.tokens.html#tokens.__getitem__',
                                                                                     'onprem/hf/data/tokens.py'),
                                       'onprem.hf.data.tokens.Tokens.__init__': ( 'hf.data.tokens.html#tokens.__init__',
                                                                                  'onprem/hf/data/tokens.py'),
                                       'onprem.hf.data.tokens.Tokens.__len__': ( 'hf.data.tokens.html#tokens.__len__',
                                                                                 'onprem/hf/data/tokens.py')},
            'onprem.hf.models.models': { 'onprem.hf.models.models.Models': ('hf.models.models.html#models', 'onprem/hf/models/models.py'),
                                         'onprem.hf.models.models.Models.checklength': ( 'hf.models.models.html#models.checklength',
                                                                                         'onprem/hf/models/models.py'),
                                         'onprem.hf.models.models.Models.device': ( 'hf.models.models.html#models.device',
                                                                                    'onprem/hf/models/models.py'),
                                         'onprem.hf.models.models.Models.deviceid': ( 'hf.models.models.html#models.deviceid',
                                                                                      'onprem/hf/models/models.py'),
                                         'onprem.hf.models.models.Models.finddevice': ( 'hf.models.models.html#models.finddevice',
                                                                                        'onprem/hf/models/models.py'),
                                         'onprem.hf.models.models.Models.hasaccelerator': ( 'hf.models.models.html#models.hasaccelerator',
                                                                                            'onprem/hf/models/models.py'),
                                         'onprem.hf.models.models.Models.hasmpsdevice': ( 'hf.models.models.html#models.hasmpsdevice',
                                                                                          'onprem/hf/models/models.py'),
                                         'onprem.hf.models.models.Models.load': ( 'hf.models.models.html#models.load',
                                                                                  'onprem/hf/models/models.py'),
                                         'onprem.hf.models.models.Models.maxlength': ( 'hf.models.models.html#models.maxlength',
                                                                                       'onprem/hf/models/models.py'),
                                         'onprem.hf.models.models.Models.reference': ( 'hf.models.models.html#models.reference',
                                                                                       'onprem/hf/models/models.py'),
                                         'onprem.hf.models.models.Models.task': ( 'hf.models.models.html#models.task',
                                                                                  'onprem/hf/models/models.py'),
                                         'onprem.hf.models.models.Models.tokenizer': ( 'hf.models.models.html#models.tokenizer',
                                                                                       'onprem/hf/models/models.py')},
            'onprem.hf.models.onnx': { 'onprem.hf.models.onnx.OnnxConfig': ('hf.models.onnx.html#onnxconfig', 'onprem/hf/models/onnx.py'),
                                       'onprem.hf.models.onnx.OnnxModel': ('hf.models.onnx.html#onnxmodel', 'onprem/hf/models/onnx.py'),
                                       'onprem.hf.models.onnx.OnnxModel.__init__': ( 'hf.models.onnx.html#onnxmodel.__init__',
                                                                                     'onprem/hf/models/onnx.py'),
                                       'onprem.hf.models.onnx.OnnxModel.device': ( 'hf.models.onnx.html#onnxmodel.device',
                                                                                   'onprem/hf/models/onnx.py'),
                                       'onprem.hf.models.onnx.OnnxModel.forward': ( 'hf.models.onnx.html#onnxmodel.forward',
                                                                                    'onprem/hf/models/onnx.py'),
                                       'onprem.hf.models.onnx.OnnxModel.parse': ( 'hf.models.onnx.html#onnxmodel.parse',
                                                                                  'onprem/hf/models/onnx.py'),
                                       'onprem.hf.models.onnx.OnnxModel.providers': ( 'hf.models.onnx.html#onnxmodel.providers',
                                                                                      'onprem/hf/models/onnx.py')},
            'onprem.hf.models.pooling.base': { 'onprem.hf.models.pooling.base.Pooling': ( 'hf.models.pooling.base.html#pooling',
                                                                                          'onprem/hf/models/pooling/base.py'),
                                               'onprem.hf.models.pooling.base.Pooling.__init__': ( 'hf.models.pooling.base.html#pooling.__init__',
                                                                                                   'onprem/hf/models/pooling/base.py'),
                                               'onprem.hf.models.pooling.base.Pooling.chunk': ( 'hf.models.pooling.base.html#pooling.chunk',
                                                                                                'onprem/hf/models/pooling/base.py'),
                                               'onprem.hf.models.pooling.base.Pooling.encode': ( 'hf.models.pooling.base.html#pooling.encode',
                                                                                                 'onprem/hf/models/pooling/base.py'),
                                               'onprem.hf.models.pooling.base.Pooling.forward': ( 'hf.models.pooling.base.html#pooling.forward',
                                                                                                  'onprem/hf/models/pooling/base.py')},
            'onprem.hf.models.pooling.cls': { 'onprem.hf.models.pooling.cls.ClsPooling': ( 'hf.models.pooling.cls.html#clspooling',
                                                                                           'onprem/hf/models/pooling/cls.py'),
                                              'onprem.hf.models.pooling.cls.ClsPooling.forward': ( 'hf.models.pooling.cls.html#clspooling.forward',
                                                                                                   'onprem/hf/models/pooling/cls.py')},
            'onprem.hf.models.pooling.factory': { 'onprem.hf.models.pooling.factory.PoolingFactory': ( 'hf.models.pooling.factory.html#poolingfactory',
                                                                                                       'onprem/hf/models/pooling/factory.py'),
                                                  'onprem.hf.models.pooling.factory.PoolingFactory.create': ( 'hf.models.pooling.factory.html#poolingfactory.create',
                                                                                                              'onprem/hf/models/pooling/factory.py'),
                                                  'onprem.hf.models.pooling.factory.PoolingFactory.load': ( 'hf.models.pooling.factory.html#poolingfactory.load',
                                                                                                            'onprem/hf/models/pooling/factory.py'),
                                                  'onprem.hf.models.pooling.factory.PoolingFactory.maxlength': ( 'hf.models.pooling.factory.html#poolingfactory.maxlength',
                                                                                                                 'onprem/hf/models/pooling/factory.py'),
                                                  'onprem.hf.models.pooling.factory.PoolingFactory.method': ( 'hf.models.pooling.factory.html#poolingfactory.method',
                                                                                                              'onprem/hf/models/pooling/factory.py')},
            'onprem.hf.models.pooling.mean': { 'onprem.hf.models.pooling.mean.MeanPooling': ( 'hf.models.pooling.mean.html#meanpooling',
                                                                                              'onprem/hf/models/pooling/mean.py'),
                                               'onprem.hf.models.pooling.mean.MeanPooling.forward': ( 'hf.models.pooling.mean.html#meanpooling.forward',
                                                                                                      'onprem/hf/models/pooling/mean.py')},
            'onprem.hf.models.registry': { 'onprem.hf.models.registry.Registry': ( 'hf.models.registry.html#registry',
                                                                                   'onprem/hf/models/registry.py'),
                                           'onprem.hf.models.registry.Registry.register': ( 'hf.models.registry.html#registry.register',
                                                                                            'onprem/hf/models/registry.py')},
            'onprem.hf.models.tokendetection': { 'onprem.hf.models.tokendetection.TokenDetection': ( 'hf.models.tokendetection.html#tokendetection',
                                                                                                     'onprem/hf/models/tokendetection.py'),
                                                 'onprem.hf.models.tokendetection.TokenDetection.__init__': ( 'hf.models.tokendetection.html#tokendetection.__init__',
                                                                                                              'onprem/hf/models/tokendetection.py'),
                                                 'onprem.hf.models.tokendetection.TokenDetection.forward': ( 'hf.models.tokendetection.html#tokendetection.forward',
                                                                                                             'onprem/hf/models/tokendetection.py'),
                                                 'onprem.hf.models.tokendetection.TokenDetection.save_pretrained': ( 'hf.models.tokendetection.html#tokendetection.save_pretrained',
                                                                                                                     'onprem/hf/models/tokendetection.py')},
            'onprem.hf.tensors': { 'onprem.hf.tensors.Tensors': ('hf.tensors.html#tensors', 'onprem/hf/tensors.py'),
                                   'onprem.hf.tensors.Tensors.context': ('hf.tensors.html#tensors.context', 'onprem/hf/tensors.py'),
                                   'onprem.hf.tensors.Tensors.quantize': ('hf.tensors.html#tensors.quantize', 'onprem/hf/tensors.py'),
                                   'onprem.hf.tensors.Tensors.tensor': ('hf.tensors.html#tensors.tensor', 'onprem/hf/tensors.py')},
            'onprem.hf.train.hfonnx': { 'onprem.hf.train.hfonnx.HFOnnx': ('hf.train.hfonnx.html#hfonnx', 'onprem/hf/train/hfonnx.py'),
                                        'onprem.hf.train.hfonnx.HFOnnx.__call__': ( 'hf.train.hfonnx.html#hfonnx.__call__',
                                                                                    'onprem/hf/train/hfonnx.py'),
                                        'onprem.hf.train.hfonnx.HFOnnx.parameters': ( 'hf.train.hfonnx.html#hfonnx.parameters',
                                                                                      'onprem/hf/train/hfonnx.py'),
                                        'onprem.hf.train.hfonnx.HFOnnx.quantization': ( 'hf.train.hfonnx.html#hfonnx.quantization',
                                                                                        'onprem/hf/train/hfonnx.py'),
                                        'onprem.hf.train.hfonnx.PoolingOnnx': ( 'hf.train.hfonnx.html#poolingonnx',
                                                                                'onprem/hf/train/hfonnx.py'),
                                        'onprem.hf.train.hfonnx.PoolingOnnx.__init__': ( 'hf.train.hfonnx.html#poolingonnx.__init__',
                                                                                         'onprem/hf/train/hfonnx.py'),
                                        'onprem.hf.train.hfonnx.PoolingOnnx.forward': ( 'hf.train.hfonnx.html#poolingonnx.forward',
                                                                                        'onprem/hf/train/hfonnx.py')},
            'onprem.hf.train.hftrainer': { 'onprem.hf.train.hftrainer.HFTrainer': ( 'hf.train.hftrainer.html#hftrainer',
                                                                                    'onprem/hf/train/hftrainer.py'),
                                           'onprem.hf.train.hftrainer.HFTrainer.__call__': ( 'hf.train.hftrainer.html#hftrainer.__call__',
                                                                                             'onprem/hf/train/hftrainer.py'),
                                           'onprem.hf.train.hftrainer.HFTrainer.load': ( 'hf.train.hftrainer.html#hftrainer.load',
                                                                                         'onprem/hf/train/hftrainer.py'),
                                           'onprem.hf.train.hftrainer.HFTrainer.lora': ( 'hf.train.hftrainer.html#hftrainer.lora',
                                                                                         'onprem/hf/train/hftrainer.py'),
                                           'onprem.hf.train.hftrainer.HFTrainer.loratask': ( 'hf.train.hftrainer.html#hftrainer.loratask',
                                                                                             'onprem/hf/train/hftrainer.py'),
                                           'onprem.hf.train.hftrainer.HFTrainer.model': ( 'hf.train.hftrainer.html#hftrainer.model',
                                                                                          'onprem/hf/train/hftrainer.py'),
                                           'onprem.hf.train.hftrainer.HFTrainer.parse': ( 'hf.train.hftrainer.html#hftrainer.parse',
                                                                                          'onprem/hf/train/hftrainer.py'),
                                           'onprem.hf.train.hftrainer.HFTrainer.peft': ( 'hf.train.hftrainer.html#hftrainer.peft',
                                                                                         'onprem/hf/train/hftrainer.py'),
                                           'onprem.hf.train.hftrainer.HFTrainer.prepare': ( 'hf.train.hftrainer.html#hftrainer.prepare',
                                                                                            'onprem/hf/train/hftrainer.py'),
                                           'onprem.hf.train.hftrainer.HFTrainer.quantization': ( 'hf.train.hftrainer.html#hftrainer.quantization',
                                                                                                 'onprem/hf/train/hftrainer.py'),
                                           'onprem.hf.train.hftrainer.TrainingArguments': ( 'hf.train.hftrainer.html#trainingarguments',
                                                                                            'onprem/hf/train/hftrainer.py'),
                                           'onprem.hf.train.hftrainer.TrainingArguments.should_save': ( 'hf.train.hftrainer.html#trainingarguments.should_save',
                                                                                                        'onprem/hf/train/hftrainer.py')},
            'onprem.hf.train.mlonnx': { 'onprem.hf.train.mlonnx.MLOnnx': ('hf.train.mlonnx.html#mlonnx', 'onprem/hf/train/mlonnx.py'),
                                        'onprem.hf.train.mlonnx.MLOnnx.__call__': ( 'hf.train.mlonnx.html#mlonnx.__call__',
                                                                                    'onprem/hf/train/mlonnx.py'),
                                        'onprem.hf.train.mlonnx.MLOnnx.__init__': ( 'hf.train.mlonnx.html#mlonnx.__init__',
                                                                                    'onprem/hf/train/mlonnx.py')},
            'onprem.ingest.base': { 'onprem.ingest.base.MyElmLoader': ('ingest.base.html#myelmloader', 'onprem/ingest/base.py'),
                                    'onprem.ingest.base.MyElmLoader.load': ('ingest.base.html#myelmloader.load', 'onprem/ingest/base.py'),
                                    'onprem.ingest.base.MyUnstructuredPDFLoader': ( 'ingest.base.html#myunstructuredpdfloader',
                                                                                    'onprem/ingest/base.py'),
                                    'onprem.ingest.base.MyUnstructuredPDFLoader.load': ( 'ingest.base.html#myunstructuredpdfloader.load',
                                                                                         'onprem/ingest/base.py'),
                                    'onprem.ingest.base.PDF2MarkdownLoader': ( 'ingest.base.html#pdf2markdownloader',
                                                                               'onprem/ingest/base.py'),
                                    'onprem.ingest.base.PDF2MarkdownLoader.load': ( 'ingest.base.html#pdf2markdownloader.load',
                                                                                    'onprem/ingest/base.py'),
                                    'onprem.ingest.base.VectorStore': ('ingest.base.html#vectorstore', 'onprem/ingest/base.py'),
                                    'onprem.ingest.base.VectorStore.add_documents': ( 'ingest.base.html#vectorstore.add_documents',
                                                                                      'onprem/ingest/base.py'),
                                    'onprem.ingest.base.VectorStore.check': ('ingest.base.html#vectorstore.check', 'onprem/ingest/base.py'),
                                    'onprem.ingest.base.VectorStore.erase': ('ingest.base.html#vectorstore.erase', 'onprem/ingest/base.py'),
                                    'onprem.ingest.base.VectorStore.exists': ( 'ingest.base.html#vectorstore.exists',
                                                                               'onprem/ingest/base.py'),
                                    'onprem.ingest.base.VectorStore.get_all_docs': ( 'ingest.base.html#vectorstore.get_all_docs',
                                                                                     'onprem/ingest/base.py'),
                                    'onprem.ingest.base.VectorStore.get_db': ( 'ingest.base.html#vectorstore.get_db',
                                                                               'onprem/ingest/base.py'),
                                    'onprem.ingest.base.VectorStore.get_doc': ( 'ingest.base.html#vectorstore.get_doc',
                                                                                'onprem/ingest/base.py'),
                                    'onprem.ingest.base.VectorStore.get_embedding_model': ( 'ingest.base.html#vectorstore.get_embedding_model',
                                                                                            'onprem/ingest/base.py'),
                                    'onprem.ingest.base.VectorStore.get_size': ( 'ingest.base.html#vectorstore.get_size',
                                                                                 'onprem/ingest/base.py'),
                                    'onprem.ingest.base.VectorStore.ingest': ( 'ingest.base.html#vectorstore.ingest',
                                                                               'onprem/ingest/base.py'),
                                    'onprem.ingest.base.VectorStore.init_embedding_model': ( 'ingest.base.html#vectorstore.init_embedding_model',
                                                                                             'onprem/ingest/base.py'),
                                    'onprem.ingest.base.VectorStore.query': ('ingest.base.html#vectorstore.query', 'onprem/ingest/base.py'),
                                    'onprem.ingest.base.VectorStore.remove_document': ( 'ingest.base.html#vectorstore.remove_document',
                                                                                        'onprem/ingest/base.py'),
                                    'onprem.ingest.base.VectorStore.semantic_search': ( 'ingest.base.html#vectorstore.semantic_search',
                                                                                        'onprem/ingest/base.py'),
                                    'onprem.ingest.base.VectorStore.update_documents': ( 'ingest.base.html#vectorstore.update_documents',
                                                                                         'onprem/ingest/base.py'),
                                    'onprem.ingest.base._PyMuPDFLoader': ('ingest.base.html#_pymupdfloader', 'onprem/ingest/base.py'),
                                    'onprem.ingest.base._PyMuPDFLoader.load': ( 'ingest.base.html#_pymupdfloader.load',
                                                                                'onprem/ingest/base.py'),
                                    'onprem.ingest.base._apply_file_callables': ( 'ingest.base.html#_apply_file_callables',
                                                                                  'onprem/ingest/base.py'),
                                    'onprem.ingest.base._apply_text_callables': ( 'ingest.base.html#_apply_text_callables',
                                                                                  'onprem/ingest/base.py'),
                                    'onprem.ingest.base._ignore_file': ('ingest.base.html#_ignore_file', 'onprem/ingest/base.py'),
                                    'onprem.ingest.base._update_metadata': ('ingest.base.html#_update_metadata', 'onprem/ingest/base.py'),
                                    'onprem.ingest.base.batchify_chunks': ('ingest.base.html#batchify_chunks', 'onprem/ingest/base.py'),
                                    'onprem.ingest.base.chunk_documents': ('ingest.base.html#chunk_documents', 'onprem/ingest/base.py'),
                                    'onprem.ingest.base.does_vectorstore_exist': ( 'ingest.base.html#does_vectorstore_exist',
                                                                                   'onprem/ingest/base.py'),
                                    'onprem.ingest.base.load_documents': ('ingest.base.html#load_documents', 'onprem/ingest/base.py'),
                                    'onprem.ingest.base.load_single_document': ( 'ingest.base.html#load_single_document',
                                                                                 'onprem/ingest/base.py'),
                                    'onprem.ingest.base.process_folder': ('ingest.base.html#process_folder', 'onprem/ingest/base.py')},
            'onprem.ingest.helpers': { 'onprem.ingest.helpers.ParagraphTextSplitter': ( 'ingest.helpers.html#paragraphtextsplitter',
                                                                                        'onprem/ingest/helpers.py'),
                                       'onprem.ingest.helpers.ParagraphTextSplitter.__init__': ( 'ingest.helpers.html#paragraphtextsplitter.__init__',
                                                                                                 'onprem/ingest/helpers.py'),
                                       'onprem.ingest.helpers.ParagraphTextSplitter._split_large_paragraph': ( 'ingest.helpers.html#paragraphtextsplitter._split_large_paragraph',
                                                                                                               'onprem/ingest/helpers.py'),
                                       'onprem.ingest.helpers.ParagraphTextSplitter.split_text': ( 'ingest.helpers.html#paragraphtextsplitter.split_text',
                                                                                                   'onprem/ingest/helpers.py'),
                                       'onprem.ingest.helpers._apply_file_callables': ( 'ingest.helpers.html#_apply_file_callables',
                                                                                        'onprem/ingest/helpers.py'),
                                       'onprem.ingest.helpers.clean_text': ('ingest.helpers.html#clean_text', 'onprem/ingest/helpers.py'),
                                       'onprem.ingest.helpers.create_document': ( 'ingest.helpers.html#create_document',
                                                                                  'onprem/ingest/helpers.py'),
                                       'onprem.ingest.helpers.date2iso': ('ingest.helpers.html#date2iso', 'onprem/ingest/helpers.py'),
                                       'onprem.ingest.helpers.doc_from_dict': ( 'ingest.helpers.html#doc_from_dict',
                                                                                'onprem/ingest/helpers.py'),
                                       'onprem.ingest.helpers.extract_extension': ( 'ingest.helpers.html#extract_extension',
                                                                                    'onprem/ingest/helpers.py'),
                                       'onprem.ingest.helpers.extract_file_dates': ( 'ingest.helpers.html#extract_file_dates',
                                                                                     'onprem/ingest/helpers.py'),
                                       'onprem.ingest.helpers.extract_file_metadata': ( 'ingest.helpers.html#extract_file_metadata',
                                                                                        'onprem/ingest/helpers.py'),
                                       'onprem.ingest.helpers.extract_files': ( 'ingest.helpers.html#extract_files',
                                                                                'onprem/ingest/helpers.py'),
                                       'onprem.ingest.helpers.extract_mimetype': ( 'ingest.helpers.html#extract_mimetype',
                                                                                   'onprem/ingest/helpers.py'),
                                       'onprem.ingest.helpers.extract_tables': ( 'ingest.helpers.html#extract_tables',
                                                                                 'onprem/ingest/helpers.py'),
                                       'onprem.ingest.helpers.get_mimetype': ( 'ingest.helpers.html#get_mimetype',
                                                                               'onprem/ingest/helpers.py'),
                                       'onprem.ingest.helpers.includes_caption': ( 'ingest.helpers.html#includes_caption',
                                                                                   'onprem/ingest/helpers.py'),
                                       'onprem.ingest.helpers.is_random_plaintext': ( 'ingest.helpers.html#is_random_plaintext',
                                                                                      'onprem/ingest/helpers.py'),
                                       'onprem.ingest.helpers.iso2date': ('ingest.helpers.html#iso2date', 'onprem/ingest/helpers.py'),
                                       'onprem.ingest.helpers.md5sum': ('ingest.helpers.html#md5sum', 'onprem/ingest/helpers.py'),
                                       'onprem.ingest.helpers.set_metadata_defaults': ( 'ingest.helpers.html#set_metadata_defaults',
                                                                                        'onprem/ingest/helpers.py')},
            'onprem.ingest.pdftables': { 'onprem.ingest.pdftables.PDFTables': ( 'ingest.pdftables.html#pdftables',
                                                                                'onprem/ingest/pdftables.py'),
                                         'onprem.ingest.pdftables.PDFTables.__init__': ( 'ingest.pdftables.html#pdftables.__init__',
                                                                                         'onprem/ingest/pdftables.py'),
                                         'onprem.ingest.pdftables.PDFTables.from_file': ( 'ingest.pdftables.html#pdftables.from_file',
                                                                                          'onprem/ingest/pdftables.py'),
                                         'onprem.ingest.pdftables.PDFTables.get_captions': ( 'ingest.pdftables.html#pdftables.get_captions',
                                                                                             'onprem/ingest/pdftables.py'),
                                         'onprem.ingest.pdftables.PDFTables.get_markdown_tables': ( 'ingest.pdftables.html#pdftables.get_markdown_tables',
                                                                                                    'onprem/ingest/pdftables.py'),
                                         'onprem.ingest.pdftables.PDFTables.get_tables': ( 'ingest.pdftables.html#pdftables.get_tables',
                                                                                           'onprem/ingest/pdftables.py'),
                                         'onprem.ingest.pdftables.PDFTables.to_markdown': ( 'ingest.pdftables.html#pdftables.to_markdown',
                                                                                            'onprem/ingest/pdftables.py'),
                                         'onprem.ingest.pdftables.ingest_pdf': ( 'ingest.pdftables.html#ingest_pdf',
                                                                                 'onprem/ingest/pdftables.py')},
            'onprem.ingest.stores.dense': { 'onprem.ingest.stores.dense.DenseStore': ( 'ingest.stores.dense.html#densestore',
                                                                                       'onprem/ingest/stores/dense.py'),
                                            'onprem.ingest.stores.dense.DenseStore.__init__': ( 'ingest.stores.dense.html#densestore.__init__',
                                                                                                'onprem/ingest/stores/dense.py'),
                                            'onprem.ingest.stores.dense.DenseStore._convert_to_dict': ( 'ingest.stores.dense.html#densestore._convert_to_dict',
                                                                                                        'onprem/ingest/stores/dense.py'),
                                            'onprem.ingest.stores.dense.DenseStore.add_documents': ( 'ingest.stores.dense.html#densestore.add_documents',
                                                                                                     'onprem/ingest/stores/dense.py'),
                                            'onprem.ingest.stores.dense.DenseStore.erase': ( 'ingest.stores.dense.html#densestore.erase',
                                                                                             'onprem/ingest/stores/dense.py'),
                                            'onprem.ingest.stores.dense.DenseStore.exists': ( 'ingest.stores.dense.html#densestore.exists',
                                                                                              'onprem/ingest/stores/dense.py'),
                                            'onprem.ingest.stores.dense.DenseStore.get_all_docs': ( 'ingest.stores.dense.html#densestore.get_all_docs',
                                                                                                    'onprem/ingest/stores/dense.py'),
                                            'onprem.ingest.stores.dense.DenseStore.get_db': ( 'ingest.stores.dense.html#densestore.get_db',
                                                                                              'onprem/ingest/stores/dense.py'),
                                            'onprem.ingest.stores.dense.DenseStore.get_doc': ( 'ingest.stores.dense.html#densestore.get_doc',
                                                                                               'onprem/ingest/stores/dense.py'),
                                            'onprem.ingest.stores.dense.DenseStore.get_size': ( 'ingest.stores.dense.html#densestore.get_size',
                                                                                                'onprem/ingest/stores/dense.py'),
                                            'onprem.ingest.stores.dense.DenseStore.optimize_for_search': ( 'ingest.stores.dense.html#densestore.optimize_for_search',
                                                                                                           'onprem/ingest/stores/dense.py'),
                                            'onprem.ingest.stores.dense.DenseStore.query': ( 'ingest.stores.dense.html#densestore.query',
                                                                                             'onprem/ingest/stores/dense.py'),
                                            'onprem.ingest.stores.dense.DenseStore.remove_document': ( 'ingest.stores.dense.html#densestore.remove_document',
                                                                                                       'onprem/ingest/stores/dense.py'),
                                            'onprem.ingest.stores.dense.DenseStore.remove_source': ( 'ingest.stores.dense.html#densestore.remove_source',
                                                                                                     'onprem/ingest/stores/dense.py'),
                                            'onprem.ingest.stores.dense.DenseStore.semantic_search': ( 'ingest.stores.dense.html#densestore.semantic_search',
                                                                                                       'onprem/ingest/stores/dense.py'),
                                            'onprem.ingest.stores.dense.DenseStore.update_documents': ( 'ingest.stores.dense.html#densestore.update_documents',
                                                                                                        'onprem/ingest/stores/dense.py')},
            'onprem.ingest.stores.dual': { 'onprem.ingest.stores.dual.DualStore': ( 'ingest.stores.dual.html#dualstore',
                                                                                    'onprem/ingest/stores/dual.py'),
                                           'onprem.ingest.stores.dual.DualStore.__init__': ( 'ingest.stores.dual.html#dualstore.__init__',
                                                                                             'onprem/ingest/stores/dual.py'),
                                           'onprem.ingest.stores.dual.DualStore.add_documents': ( 'ingest.stores.dual.html#dualstore.add_documents',
                                                                                                  'onprem/ingest/stores/dual.py'),
                                           'onprem.ingest.stores.dual.DualStore.erase': ( 'ingest.stores.dual.html#dualstore.erase',
                                                                                          'onprem/ingest/stores/dual.py'),
                                           'onprem.ingest.stores.dual.DualStore.exists': ( 'ingest.stores.dual.html#dualstore.exists',
                                                                                           'onprem/ingest/stores/dual.py'),
                                           'onprem.ingest.stores.dual.DualStore.get_all_docs': ( 'ingest.stores.dual.html#dualstore.get_all_docs',
                                                                                                 'onprem/ingest/stores/dual.py'),
                                           'onprem.ingest.stores.dual.DualStore.get_db': ( 'ingest.stores.dual.html#dualstore.get_db',
                                                                                           'onprem/ingest/stores/dual.py'),
                                           'onprem.ingest.stores.dual.DualStore.get_dense_db': ( 'ingest.stores.dual.html#dualstore.get_dense_db',
                                                                                                 'onprem/ingest/stores/dual.py'),
                                           'onprem.ingest.stores.dual.DualStore.get_doc': ( 'ingest.stores.dual.html#dualstore.get_doc',
                                                                                            'onprem/ingest/stores/dual.py'),
                                           'onprem.ingest.stores.dual.DualStore.get_size': ( 'ingest.stores.dual.html#dualstore.get_size',
                                                                                             'onprem/ingest/stores/dual.py'),
                                           'onprem.ingest.stores.dual.DualStore.get_sparse_db': ( 'ingest.stores.dual.html#dualstore.get_sparse_db',
                                                                                                  'onprem/ingest/stores/dual.py'),
                                           'onprem.ingest.stores.dual.DualStore.keyword_search': ( 'ingest.stores.dual.html#dualstore.keyword_search',
                                                                                                   'onprem/ingest/stores/dual.py'),
                                           'onprem.ingest.stores.dual.DualStore.optimize_for_search': ( 'ingest.stores.dual.html#dualstore.optimize_for_search',
                                                                                                        'onprem/ingest/stores/dual.py'),
                                           'onprem.ingest.stores.dual.DualStore.query': ( 'ingest.stores.dual.html#dualstore.query',
                                                                                          'onprem/ingest/stores/dual.py'),
                                           'onprem.ingest.stores.dual.DualStore.remove_document': ( 'ingest.stores.dual.html#dualstore.remove_document',
                                                                                                    'onprem/ingest/stores/dual.py'),
                                           'onprem.ingest.stores.dual.DualStore.remove_source': ( 'ingest.stores.dual.html#dualstore.remove_source',
                                                                                                  'onprem/ingest/stores/dual.py'),
                                           'onprem.ingest.stores.dual.DualStore.semantic_search': ( 'ingest.stores.dual.html#dualstore.semantic_search',
                                                                                                    'onprem/ingest/stores/dual.py'),
                                           'onprem.ingest.stores.dual.DualStore.update_documents': ( 'ingest.stores.dual.html#dualstore.update_documents',
                                                                                                     'onprem/ingest/stores/dual.py')},
            'onprem.ingest.stores.sparse': { 'onprem.ingest.stores.sparse.SparseStore': ( 'ingest.stores.sparse.html#sparsestore',
                                                                                          'onprem/ingest/stores/sparse.py'),
                                             'onprem.ingest.stores.sparse.SparseStore.__init__': ( 'ingest.stores.sparse.html#sparsestore.__init__',
                                                                                                   'onprem/ingest/stores/sparse.py'),
                                             'onprem.ingest.stores.sparse.SparseStore._preprocess_query': ( 'ingest.stores.sparse.html#sparsestore._preprocess_query',
                                                                                                            'onprem/ingest/stores/sparse.py'),
                                             'onprem.ingest.stores.sparse.SparseStore.add_documents': ( 'ingest.stores.sparse.html#sparsestore.add_documents',
                                                                                                        'onprem/ingest/stores/sparse.py'),
                                             'onprem.ingest.stores.sparse.SparseStore.delete_by_prefix': ( 'ingest.stores.sparse.html#sparsestore.delete_by_prefix',
                                                                                                           'onprem/ingest/stores/sparse.py'),
                                             'onprem.ingest.stores.sparse.SparseStore.doc2dict': ( 'ingest.stores.sparse.html#sparsestore.doc2dict',
                                                                                                   'onprem/ingest/stores/sparse.py'),
                                             'onprem.ingest.stores.sparse.SparseStore.erase': ( 'ingest.stores.sparse.html#sparsestore.erase',
                                                                                                'onprem/ingest/stores/sparse.py'),
                                             'onprem.ingest.stores.sparse.SparseStore.exists': ( 'ingest.stores.sparse.html#sparsestore.exists',
                                                                                                 'onprem/ingest/stores/sparse.py'),
                                             'onprem.ingest.stores.sparse.SparseStore.get_all_docs': ( 'ingest.stores.sparse.html#sparsestore.get_all_docs',
                                                                                                       'onprem/ingest/stores/sparse.py'),
                                             'onprem.ingest.stores.sparse.SparseStore.get_db': ( 'ingest.stores.sparse.html#sparsestore.get_db',
                                                                                                 'onprem/ingest/stores/sparse.py'),
                                             'onprem.ingest.stores.sparse.SparseStore.get_doc': ( 'ingest.stores.sparse.html#sparsestore.get_doc',
                                                                                                  'onprem/ingest/stores/sparse.py'),
                                             'onprem.ingest.stores.sparse.SparseStore.get_size': ( 'ingest.stores.sparse.html#sparsestore.get_size',
                                                                                                   'onprem/ingest/stores/sparse.py'),
                                             'onprem.ingest.stores.sparse.SparseStore.index_exists_in': ( 'ingest.stores.sparse.html#sparsestore.index_exists_in',
                                                                                                          'onprem/ingest/stores/sparse.py'),
                                             'onprem.ingest.stores.sparse.SparseStore.initialize_index': ( 'ingest.stores.sparse.html#sparsestore.initialize_index',
                                                                                                           'onprem/ingest/stores/sparse.py'),
                                             'onprem.ingest.stores.sparse.SparseStore.normalize_text': ( 'ingest.stores.sparse.html#sparsestore.normalize_text',
                                                                                                         'onprem/ingest/stores/sparse.py'),
                                             'onprem.ingest.stores.sparse.SparseStore.query': ( 'ingest.stores.sparse.html#sparsestore.query',
                                                                                                'onprem/ingest/stores/sparse.py'),
                                             'onprem.ingest.stores.sparse.SparseStore.remove_document': ( 'ingest.stores.sparse.html#sparsestore.remove_document',
                                                                                                          'onprem/ingest/stores/sparse.py'),
                                             'onprem.ingest.stores.sparse.SparseStore.remove_source': ( 'ingest.stores.sparse.html#sparsestore.remove_source',
                                                                                                        'onprem/ingest/stores/sparse.py'),
                                             'onprem.ingest.stores.sparse.SparseStore.semantic_search': ( 'ingest.stores.sparse.html#sparsestore.semantic_search',
                                                                                                          'onprem/ingest/stores/sparse.py'),
                                             'onprem.ingest.stores.sparse.SparseStore.update_documents': ( 'ingest.stores.sparse.html#sparsestore.update_documents',
                                                                                                           'onprem/ingest/stores/sparse.py'),
                                             'onprem.ingest.stores.sparse.default_schema': ( 'ingest.stores.sparse.html#default_schema',
                                                                                             'onprem/ingest/stores/sparse.py')},
            'onprem.llm.base': { 'onprem.llm.base.LLM': ('llm.base.html#llm', 'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM.__init__': ('llm.base.html#llm.__init__', 'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM._ask': ('llm.base.html#llm._ask', 'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM._format_image_prompt': ( 'llm.base.html#llm._format_image_prompt',
                                                                               'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM._format_pydantic_prompt': ( 'llm.base.html#llm._format_pydantic_prompt',
                                                                                  'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM.ask': ('llm.base.html#llm.ask', 'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM.chat': ('llm.base.html#llm.chat', 'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM.check_model': ('llm.base.html#llm.check_model', 'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM.download_model': ('llm.base.html#llm.download_model', 'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM.get_store_type': ('llm.base.html#llm.get_store_type', 'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM.ingest': ('llm.base.html#llm.ingest', 'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM.is_azure': ('llm.base.html#llm.is_azure', 'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM.is_dense_store': ('llm.base.html#llm.is_dense_store', 'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM.is_dual_store': ('llm.base.html#llm.is_dual_store', 'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM.is_hf': ('llm.base.html#llm.is_hf', 'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM.is_llamacpp': ('llm.base.html#llm.is_llamacpp', 'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM.is_local': ('llm.base.html#llm.is_local', 'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM.is_local_api': ('llm.base.html#llm.is_local_api', 'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM.is_openai_model': ('llm.base.html#llm.is_openai_model', 'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM.is_sparse_store': ('llm.base.html#llm.is_sparse_store', 'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM.load_chatbot': ('llm.base.html#llm.load_chatbot', 'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM.load_llm': ('llm.base.html#llm.load_llm', 'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM.load_vectordb': ('llm.base.html#llm.load_vectordb', 'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM.load_vectorstore': ('llm.base.html#llm.load_vectorstore', 'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM.optimize_vectorstore_for_search': ( 'llm.base.html#llm.optimize_vectorstore_for_search',
                                                                                          'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM.process_service': ('llm.base.html#llm.process_service', 'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM.prompt': ('llm.base.html#llm.prompt', 'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM.pydantic_prompt': ('llm.base.html#llm.pydantic_prompt', 'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM.query': ('llm.base.html#llm.query', 'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM.set_store_type': ('llm.base.html#llm.set_store_type', 'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM.update_max_tokens': ('llm.base.html#llm.update_max_tokens', 'onprem/llm/base.py'),
                                 'onprem.llm.base.LLM.update_stop': ('llm.base.html#llm.update_stop', 'onprem/llm/base.py')},
            'onprem.llm.helpers': { 'onprem.llm.helpers.TableSummary': ('llm.helpers.html#tablesummary', 'onprem/llm/helpers.py'),
                                    'onprem.llm.helpers.Title': ('llm.helpers.html#title', 'onprem/llm/helpers.py'),
                                    'onprem.llm.helpers._marshal_llm_to_json': ( 'llm.helpers.html#_marshal_llm_to_json',
                                                                                 'onprem/llm/helpers.py'),
                                    'onprem.llm.helpers.caption_table_text': ( 'llm.helpers.html#caption_table_text',
                                                                               'onprem/llm/helpers.py'),
                                    'onprem.llm.helpers.decompose_question': ( 'llm.helpers.html#decompose_question',
                                                                               'onprem/llm/helpers.py'),
                                    'onprem.llm.helpers.extract_json': ('llm.helpers.html#extract_json', 'onprem/llm/helpers.py'),
                                    'onprem.llm.helpers.extract_title': ('llm.helpers.html#extract_title', 'onprem/llm/helpers.py'),
                                    'onprem.llm.helpers.needs_followup': ('llm.helpers.html#needs_followup', 'onprem/llm/helpers.py'),
                                    'onprem.llm.helpers.parse_code_markdown': ( 'llm.helpers.html#parse_code_markdown',
                                                                                'onprem/llm/helpers.py'),
                                    'onprem.llm.helpers.parse_json_markdown': ( 'llm.helpers.html#parse_json_markdown',
                                                                                'onprem/llm/helpers.py'),
                                    'onprem.llm.helpers.summarize_tables': ('llm.helpers.html#summarize_tables', 'onprem/llm/helpers.py'),
                                    'onprem.llm.helpers.truncate_prompt': ('llm.helpers.html#truncate_prompt', 'onprem/llm/helpers.py')},
            'onprem.pipelines.agent.base': { 'onprem.pipelines.agent.base.Agent': ( 'pipelines.agent.base.html#agent',
                                                                                    'onprem/pipelines/agent/base.py'),
                                             'onprem.pipelines.agent.base.Agent.__init__': ( 'pipelines.agent.base.html#agent.__init__',
                                                                                             'onprem/pipelines/agent/base.py'),
                                             'onprem.pipelines.agent.base.Agent._init_agent': ( 'pipelines.agent.base.html#agent._init_agent',
                                                                                                'onprem/pipelines/agent/base.py'),
                                             'onprem.pipelines.agent.base.Agent.add_default_tool': ( 'pipelines.agent.base.html#agent.add_default_tool',
                                                                                                     'onprem/pipelines/agent/base.py'),
                                             'onprem.pipelines.agent.base.Agent.add_function_tool': ( 'pipelines.agent.base.html#agent.add_function_tool',
                                                                                                      'onprem/pipelines/agent/base.py'),
                                             'onprem.pipelines.agent.base.Agent.add_mcp_tool': ( 'pipelines.agent.base.html#agent.add_mcp_tool',
                                                                                                 'onprem/pipelines/agent/base.py'),
                                             'onprem.pipelines.agent.base.Agent.add_python_tool': ( 'pipelines.agent.base.html#agent.add_python_tool',
                                                                                                    'onprem/pipelines/agent/base.py'),
                                             'onprem.pipelines.agent.base.Agent.add_tool': ( 'pipelines.agent.base.html#agent.add_tool',
                                                                                             'onprem/pipelines/agent/base.py'),
                                             'onprem.pipelines.agent.base.Agent.add_vectorstore_tool': ( 'pipelines.agent.base.html#agent.add_vectorstore_tool',
                                                                                                         'onprem/pipelines/agent/base.py'),
                                             'onprem.pipelines.agent.base.Agent.add_websearch_tool': ( 'pipelines.agent.base.html#agent.add_websearch_tool',
                                                                                                       'onprem/pipelines/agent/base.py'),
                                             'onprem.pipelines.agent.base.Agent.add_webview_tool': ( 'pipelines.agent.base.html#agent.add_webview_tool',
                                                                                                     'onprem/pipelines/agent/base.py'),
                                             'onprem.pipelines.agent.base.Agent.run': ( 'pipelines.agent.base.html#agent.run',
                                                                                        'onprem/pipelines/agent/base.py')},
            'onprem.pipelines.agent.model': { 'onprem.pipelines.agent.model.AgentModel': ( 'pipelines.agent.model.html#agentmodel',
                                                                                           'onprem/pipelines/agent/model.py'),
                                              'onprem.pipelines.agent.model.AgentModel.__init__': ( 'pipelines.agent.model.html#agentmodel.__init__',
                                                                                                    'onprem/pipelines/agent/model.py'),
                                              'onprem.pipelines.agent.model.AgentModel.clean': ( 'pipelines.agent.model.html#agentmodel.clean',
                                                                                                 'onprem/pipelines/agent/model.py'),
                                              'onprem.pipelines.agent.model.AgentModel.generate': ( 'pipelines.agent.model.html#agentmodel.generate',
                                                                                                    'onprem/pipelines/agent/model.py'),
                                              'onprem.pipelines.agent.model.AgentModel.parameters': ( 'pipelines.agent.model.html#agentmodel.parameters',
                                                                                                      'onprem/pipelines/agent/model.py')},
            'onprem.pipelines.agent.tools': { 'onprem.pipelines.agent.tools.FunctionTool': ( 'pipelines.agent.tools.html#functiontool',
                                                                                             'onprem/pipelines/agent/tools.py'),
                                              'onprem.pipelines.agent.tools.FunctionTool.__init__': ( 'pipelines.agent.tools.html#functiontool.__init__',
                                                                                                      'onprem/pipelines/agent/tools.py'),
                                              'onprem.pipelines.agent.tools.FunctionTool.forward': ( 'pipelines.agent.tools.html#functiontool.forward',
                                                                                                     'onprem/pipelines/agent/tools.py'),
                                              'onprem.pipelines.agent.tools.VectorStoreTool': ( 'pipelines.agent.tools.html#vectorstoretool',
                                                                                                'onprem/pipelines/agent/tools.py'),
                                              'onprem.pipelines.agent.tools.VectorStoreTool.__init__': ( 'pipelines.agent.tools.html#vectorstoretool.__init__',
                                                                                                         'onprem/pipelines/agent/tools.py'),
                                              'onprem.pipelines.agent.tools.VectorStoreTool.forward': ( 'pipelines.agent.tools.html#vectorstoretool.forward',
                                                                                                        'onprem/pipelines/agent/tools.py'),
                                              'onprem.pipelines.agent.tools.createtool': ( 'pipelines.agent.tools.html#createtool',
                                                                                           'onprem/pipelines/agent/tools.py'),
                                              'onprem.pipelines.agent.tools.fromdocs': ( 'pipelines.agent.tools.html#fromdocs',
                                                                                         'onprem/pipelines/agent/tools.py')},
            'onprem.pipelines.classifier': { 'onprem.pipelines.classifier.ClassifierBase': ( 'pipelines.classifier.html#classifierbase',
                                                                                             'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.ClassifierBase.arrays2dataset': ( 'pipelines.classifier.html#classifierbase.arrays2dataset',
                                                                                                            'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.ClassifierBase.dataset2arrays': ( 'pipelines.classifier.html#classifierbase.dataset2arrays',
                                                                                                            'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.ClassifierBase.evaluate': ( 'pipelines.classifier.html#classifierbase.evaluate',
                                                                                                      'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.ClassifierBase.explain': ( 'pipelines.classifier.html#classifierbase.explain',
                                                                                                     'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.ClassifierBase.get_explain_predictor': ( 'pipelines.classifier.html#classifierbase.get_explain_predictor',
                                                                                                                   'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.ClassifierBase.get_labels': ( 'pipelines.classifier.html#classifierbase.get_labels',
                                                                                                        'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.ClassifierBase.get_trainer': ( 'pipelines.classifier.html#classifierbase.get_trainer',
                                                                                                         'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.ClassifierBase.sample_examples': ( 'pipelines.classifier.html#classifierbase.sample_examples',
                                                                                                             'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.ClassifierBase.save': ( 'pipelines.classifier.html#classifierbase.save',
                                                                                                  'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.ClassifierBase.train': ( 'pipelines.classifier.html#classifierbase.train',
                                                                                                   'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.FewShotClassifier': ( 'pipelines.classifier.html#fewshotclassifier',
                                                                                                'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.FewShotClassifier.__init__': ( 'pipelines.classifier.html#fewshotclassifier.__init__',
                                                                                                         'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.FewShotClassifier._get_explain_predictor': ( 'pipelines.classifier.html#fewshotclassifier._get_explain_predictor',
                                                                                                                       'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.FewShotClassifier.predict': ( 'pipelines.classifier.html#fewshotclassifier.predict',
                                                                                                        'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.FewShotClassifier.predict_proba': ( 'pipelines.classifier.html#fewshotclassifier.predict_proba',
                                                                                                              'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.FewShotClassifier.save': ( 'pipelines.classifier.html#fewshotclassifier.save',
                                                                                                     'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.FewShotClassifier.train': ( 'pipelines.classifier.html#fewshotclassifier.train',
                                                                                                      'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.HFClassifier': ( 'pipelines.classifier.html#hfclassifier',
                                                                                           'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.HFClassifier.__init__': ( 'pipelines.classifier.html#hfclassifier.__init__',
                                                                                                    'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.HFClassifier._get_explain_predictor': ( 'pipelines.classifier.html#hfclassifier._get_explain_predictor',
                                                                                                                  'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.HFClassifier._get_pipeline': ( 'pipelines.classifier.html#hfclassifier._get_pipeline',
                                                                                                         'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.HFClassifier._predict': ( 'pipelines.classifier.html#hfclassifier._predict',
                                                                                                    'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.HFClassifier._tempsave': ( 'pipelines.classifier.html#hfclassifier._tempsave',
                                                                                                     'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.HFClassifier.predict': ( 'pipelines.classifier.html#hfclassifier.predict',
                                                                                                   'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.HFClassifier.predict_proba': ( 'pipelines.classifier.html#hfclassifier.predict_proba',
                                                                                                         'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.HFClassifier.save': ( 'pipelines.classifier.html#hfclassifier.save',
                                                                                                'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.HFClassifier.train': ( 'pipelines.classifier.html#hfclassifier.train',
                                                                                                 'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.SKClassifier': ( 'pipelines.classifier.html#skclassifier',
                                                                                           'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.SKClassifier.__init__': ( 'pipelines.classifier.html#skclassifier.__init__',
                                                                                                    'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.SKClassifier.predict': ( 'pipelines.classifier.html#skclassifier.predict',
                                                                                                   'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.SKClassifier.predict_proba': ( 'pipelines.classifier.html#skclassifier.predict_proba',
                                                                                                         'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.SKClassifier.save': ( 'pipelines.classifier.html#skclassifier.save',
                                                                                                'onprem/pipelines/classifier.py'),
                                             'onprem.pipelines.classifier.SKClassifier.train': ( 'pipelines.classifier.html#skclassifier.train',
                                                                                                 'onprem/pipelines/classifier.py')},
            'onprem.pipelines.extractor': { 'onprem.pipelines.extractor.Extractor': ( 'pipelines.extractor.html#extractor',
                                                                                      'onprem/pipelines/extractor.py'),
                                            'onprem.pipelines.extractor.Extractor.__init__': ( 'pipelines.extractor.html#extractor.__init__',
                                                                                               'onprem/pipelines/extractor.py'),
                                            'onprem.pipelines.extractor.Extractor.apply': ( 'pipelines.extractor.html#extractor.apply',
                                                                                            'onprem/pipelines/extractor.py')},
            'onprem.pipelines.guider': { 'onprem.pipelines.guider.Guider': ('pipelines.guider.html#guider', 'onprem/pipelines/guider.py'),
                                         'onprem.pipelines.guider.Guider.__init__': ( 'pipelines.guider.html#guider.__init__',
                                                                                      'onprem/pipelines/guider.py'),
                                         'onprem.pipelines.guider.Guider.prompt': ( 'pipelines.guider.html#guider.prompt',
                                                                                    'onprem/pipelines/guider.py')},
            'onprem.pipelines.summarizer': { 'onprem.pipelines.summarizer.Summarizer': ( 'pipelines.summarizer.html#summarizer',
                                                                                         'onprem/pipelines/summarizer.py'),
                                             'onprem.pipelines.summarizer.Summarizer.__init__': ( 'pipelines.summarizer.html#summarizer.__init__',
                                                                                                  'onprem/pipelines/summarizer.py'),
                                             'onprem.pipelines.summarizer.Summarizer._map_reduce': ( 'pipelines.summarizer.html#summarizer._map_reduce',
                                                                                                     'onprem/pipelines/summarizer.py'),
                                             'onprem.pipelines.summarizer.Summarizer._refine': ( 'pipelines.summarizer.html#summarizer._refine',
                                                                                                 'onprem/pipelines/summarizer.py'),
                                             'onprem.pipelines.summarizer.Summarizer.summarize': ( 'pipelines.summarizer.html#summarizer.summarize',
                                                                                                   'onprem/pipelines/summarizer.py'),
                                             'onprem.pipelines.summarizer.Summarizer.summarize_by_concept': ( 'pipelines.summarizer.html#summarizer.summarize_by_concept',
                                                                                                              'onprem/pipelines/summarizer.py')},
            'onprem.sk.base': { 'onprem.sk.base.decode_by_line': ('sk.base.html#decode_by_line', 'onprem/sk/base.py'),
                                'onprem.sk.base.detect_encoding': ('sk.base.html#detect_encoding', 'onprem/sk/base.py'),
                                'onprem.sk.base.detect_lang': ('sk.base.html#detect_lang', 'onprem/sk/base.py'),
                                'onprem.sk.base.get_random_colors': ('sk.base.html#get_random_colors', 'onprem/sk/base.py'),
                                'onprem.sk.base.is_chinese': ('sk.base.html#is_chinese', 'onprem/sk/base.py'),
                                'onprem.sk.base.is_nospace_lang': ('sk.base.html#is_nospace_lang', 'onprem/sk/base.py'),
                                'onprem.sk.base.split_chinese': ('sk.base.html#split_chinese', 'onprem/sk/base.py')},
            'onprem.sk.clf': { 'onprem.sk.clf.Classifier': ('sk.clf.html#classifier', 'onprem/sk/clf.py'),
                               'onprem.sk.clf.Classifier.__init__': ('sk.clf.html#classifier.__init__', 'onprem/sk/clf.py'),
                               'onprem.sk.clf.Classifier.create_model': ('sk.clf.html#classifier.create_model', 'onprem/sk/clf.py'),
                               'onprem.sk.clf.Classifier.evaluate': ('sk.clf.html#classifier.evaluate', 'onprem/sk/clf.py'),
                               'onprem.sk.clf.Classifier.fit': ('sk.clf.html#classifier.fit', 'onprem/sk/clf.py'),
                               'onprem.sk.clf.Classifier.grid_search': ('sk.clf.html#classifier.grid_search', 'onprem/sk/clf.py'),
                               'onprem.sk.clf.Classifier.load': ('sk.clf.html#classifier.load', 'onprem/sk/clf.py'),
                               'onprem.sk.clf.Classifier.load_texts_from_csv': ( 'sk.clf.html#classifier.load_texts_from_csv',
                                                                                 'onprem/sk/clf.py'),
                               'onprem.sk.clf.Classifier.load_texts_from_folder': ( 'sk.clf.html#classifier.load_texts_from_folder',
                                                                                    'onprem/sk/clf.py'),
                               'onprem.sk.clf.Classifier.predict': ('sk.clf.html#classifier.predict', 'onprem/sk/clf.py'),
                               'onprem.sk.clf.Classifier.predict_proba': ('sk.clf.html#classifier.predict_proba', 'onprem/sk/clf.py'),
                               'onprem.sk.clf.Classifier.save': ('sk.clf.html#classifier.save', 'onprem/sk/clf.py'),
                               'onprem.sk.clf.NBSVM': ('sk.clf.html#nbsvm', 'onprem/sk/clf.py'),
                               'onprem.sk.clf.NBSVM.__init__': ('sk.clf.html#nbsvm.__init__', 'onprem/sk/clf.py'),
                               'onprem.sk.clf.NBSVM._fit_binary': ('sk.clf.html#nbsvm._fit_binary', 'onprem/sk/clf.py'),
                               'onprem.sk.clf.NBSVM.fit': ('sk.clf.html#nbsvm.fit', 'onprem/sk/clf.py')},
            'onprem.sk.tm': { 'onprem.sk.tm.TopicModel': ('sk.tm.html#topicmodel', 'onprem/sk/tm.py'),
                              'onprem.sk.tm.TopicModel.__init__': ('sk.tm.html#topicmodel.__init__', 'onprem/sk/tm.py'),
                              'onprem.sk.tm.TopicModel._check_build': ('sk.tm.html#topicmodel._check_build', 'onprem/sk/tm.py'),
                              'onprem.sk.tm.TopicModel._check_model': ('sk.tm.html#topicmodel._check_model', 'onprem/sk/tm.py'),
                              'onprem.sk.tm.TopicModel._check_recommender': ('sk.tm.html#topicmodel._check_recommender', 'onprem/sk/tm.py'),
                              'onprem.sk.tm.TopicModel._check_scorer': ('sk.tm.html#topicmodel._check_scorer', 'onprem/sk/tm.py'),
                              'onprem.sk.tm.TopicModel._harden_topics': ('sk.tm.html#topicmodel._harden_topics', 'onprem/sk/tm.py'),
                              'onprem.sk.tm.TopicModel._rank_documents': ('sk.tm.html#topicmodel._rank_documents', 'onprem/sk/tm.py'),
                              'onprem.sk.tm.TopicModel.build': ('sk.tm.html#topicmodel.build', 'onprem/sk/tm.py'),
                              'onprem.sk.tm.TopicModel.filter': ('sk.tm.html#topicmodel.filter', 'onprem/sk/tm.py'),
                              'onprem.sk.tm.TopicModel.get_docs': ('sk.tm.html#topicmodel.get_docs', 'onprem/sk/tm.py'),
                              'onprem.sk.tm.TopicModel.get_doctopics': ('sk.tm.html#topicmodel.get_doctopics', 'onprem/sk/tm.py'),
                              'onprem.sk.tm.TopicModel.get_document_topic_distribution': ( 'sk.tm.html#topicmodel.get_document_topic_distribution',
                                                                                           'onprem/sk/tm.py'),
                              'onprem.sk.tm.TopicModel.get_sorted_docs': ('sk.tm.html#topicmodel.get_sorted_docs', 'onprem/sk/tm.py'),
                              'onprem.sk.tm.TopicModel.get_texts': ('sk.tm.html#topicmodel.get_texts', 'onprem/sk/tm.py'),
                              'onprem.sk.tm.TopicModel.get_topics': ('sk.tm.html#topicmodel.get_topics', 'onprem/sk/tm.py'),
                              'onprem.sk.tm.TopicModel.get_word_weights': ('sk.tm.html#topicmodel.get_word_weights', 'onprem/sk/tm.py'),
                              'onprem.sk.tm.TopicModel.predict': ('sk.tm.html#topicmodel.predict', 'onprem/sk/tm.py'),
                              'onprem.sk.tm.TopicModel.print_topics': ('sk.tm.html#topicmodel.print_topics', 'onprem/sk/tm.py'),
                              'onprem.sk.tm.TopicModel.recommend': ('sk.tm.html#topicmodel.recommend', 'onprem/sk/tm.py'),
                              'onprem.sk.tm.TopicModel.save': ('sk.tm.html#topicmodel.save', 'onprem/sk/tm.py'),
                              'onprem.sk.tm.TopicModel.score': ('sk.tm.html#topicmodel.score', 'onprem/sk/tm.py'),
                              'onprem.sk.tm.TopicModel.search': ('sk.tm.html#topicmodel.search', 'onprem/sk/tm.py'),
                              'onprem.sk.tm.TopicModel.topics': ('sk.tm.html#topicmodel.topics', 'onprem/sk/tm.py'),
                              'onprem.sk.tm.TopicModel.train': ('sk.tm.html#topicmodel.train', 'onprem/sk/tm.py'),
                              'onprem.sk.tm.TopicModel.train_recommender': ('sk.tm.html#topicmodel.train_recommender', 'onprem/sk/tm.py'),
                              'onprem.sk.tm.TopicModel.train_scorer': ('sk.tm.html#topicmodel.train_scorer', 'onprem/sk/tm.py'),
                              'onprem.sk.tm.TopicModel.visualize_documents': ( 'sk.tm.html#topicmodel.visualize_documents',
                                                                               'onprem/sk/tm.py')},
            'onprem.utils': { 'onprem.utils.SafeFormatter': ('utils.html#safeformatter', 'onprem/utils.py'),
                              'onprem.utils.SafeFormatter.__init__': ('utils.html#safeformatter.__init__', 'onprem/utils.py'),
                              'onprem.utils.SafeFormatter._replace_match': ('utils.html#safeformatter._replace_match', 'onprem/utils.py'),
                              'onprem.utils.SafeFormatter.format': ('utils.html#safeformatter.format', 'onprem/utils.py'),
                              'onprem.utils.SafeFormatter.parse': ('utils.html#safeformatter.parse', 'onprem/utils.py'),
                              'onprem.utils.batch_generator': ('utils.html#batch_generator', 'onprem/utils.py'),
                              'onprem.utils.batch_list': ('utils.html#batch_list', 'onprem/utils.py'),
                              'onprem.utils.contains_sentence': ('utils.html#contains_sentence', 'onprem/utils.py'),
                              'onprem.utils.df_to_md': ('utils.html#df_to_md', 'onprem/utils.py'),
                              'onprem.utils.download': ('utils.html#download', 'onprem/utils.py'),
                              'onprem.utils.filtered_generator': ('utils.html#filtered_generator', 'onprem/utils.py'),
                              'onprem.utils.format_string': ('utils.html#format_string', 'onprem/utils.py'),
                              'onprem.utils.get_datadir': ('utils.html#get_datadir', 'onprem/utils.py'),
                              'onprem.utils.get_models_dir': ('utils.html#get_models_dir', 'onprem/utils.py'),
                              'onprem.utils.get_template_vars': ('utils.html#get_template_vars', 'onprem/utils.py'),
                              'onprem.utils.get_webapp_dir': ('utils.html#get_webapp_dir', 'onprem/utils.py'),
                              'onprem.utils.html_to_df': ('utils.html#html_to_df', 'onprem/utils.py'),
                              'onprem.utils.md_to_df': ('utils.html#md_to_df', 'onprem/utils.py'),
                              'onprem.utils.remove_sentence': ('utils.html#remove_sentence', 'onprem/utils.py'),
                              'onprem.utils.segment': ('utils.html#segment', 'onprem/utils.py')}}}
